from django.db import models
from .compat import DJANGO16


class _PersistentModelManager(models.Manager):
    def create(self, *args, **kwargs):
        user = kwargs.pop('current_user', None)
        kwargs['created_by'] = user
        kwargs['updated_by'] = user
        instance = super(_PersistentModelManager, self).create(*args, **kwargs)
        return instance


if DJANGO16:
    class PersistentModelManager(_PersistentModelManager):
        def get_queryset(self):
            return super(PersistentModelManager, self).get_queryset().filter(deleted=False)
else:
    class PersistentModelManager(_PersistentModelManager):
        def get_query_set(self):
            return super(PersistentModelManager, self).get_query_set().filter(deleted=False)



# -*- coding: utf8 -*-
import sys
from datetime import datetime

from rest_framework.test import APITestCase, force_authenticate, APIRequestFactory

from django.utils import unittest, timezone
from django.test.utils import override_settings

from basis.compat import get_user_model
from .models import Person
from .views import BasisModelViewSet

PY3 = sys.version_info[0] == 3

if PY3:
    from unittest import mock
else:
    import mock


class TestTimeStampModel(unittest.TestCase):

    def setUp(self):
        self.user1 = get_user_model().objects.get_or_create(username="test1")[0]
        self.user2 = get_user_model().objects.get_or_create(username="test2")[0]

    def tearDown(self):
        get_user_model().objects.all().delete()
        Person.all_objects.all().delete()

    def test_datetimes(self):
        person = Person.objects.create(current_user=self.user1)

        self.assertAlmostEqual(int(person.created_at.strftime('%Y%m%d%H%M%S')),
                               int(datetime.now().strftime('%Y%m%d%H%M%S')))
        person.save()
        self.assertNotEqual(person.created_at, person.updated_at)
        self.assertAlmostEqual(int(person.updated_at.strftime('%Y%m%d%H%M%S')),
                               int(datetime.now().strftime('%Y%m%d%H%M%S')))

    @override_settings(USE_TZ=True)
    def test_datetimes_with_timezone(self):
        person = Person.objects.create(current_user=self.user1)
        self.assertEqual(person.created_at.tzinfo, timezone.utc)
        self.assertEqual(person.updated_at.tzinfo, timezone.utc)

        self.assertAlmostEqual(int(person.created_at.strftime('%Y%m%d%H%M%S')),
                               int(timezone.now().strftime('%Y%m%d%H%M%S')))
        person.save()
        self.assertNotEqual(person.created_at, person.updated_at)
        self.assertAlmostEqual(int(person.updated_at.strftime('%Y%m%d%H%M%S')),
                               int(timezone.now().strftime('%Y%m%d%H%M%S')))


class TestPersistentModel(unittest.TestCase):

    def setUp(self):
        self.user1 = get_user_model().objects.get_or_create(username='test1')[0]
        self.user2 = get_user_model().objects.get_or_create(username='test2')[0]

    def tearDown(self):
        get_user_model().objects.all().delete()
        Person.all_objects.all().delete()

    def test_delete_person(self):
        person = Person.objects.create(current_user=self.user1)

        self.assertEqual(Person.objects.all().count(), 1)
        person.delete()
        self.assertEqual(Person.objects.all().count(), 0)
        self.assertEqual(Person.all_objects.all().count(), 1)

        person.restore()
        self.assertEqual(Person.objects.all().count(), 1)
        Person.all_objects.all()[0].delete()
        self.assertEqual(Person.objects.all().count(), 0)
        self.assertEqual(Person.all_objects.all().count(), 1)

    def test_force_delete(self):
        person = Person.objects.create(current_user=self.user1)
        person.delete(force=True)
        self.assertEqual(Person.all_objects.count(), 0)

    @mock.patch('django.db.models.base.Model.delete')
    def test_delete_with_using(self, mock_delete):
        person = Person.objects.create(current_user=self.user1)
        person.delete(force=True, using='other_db')
        mock_delete.assert_called_with('other_db')


class TestBasisModel(unittest.TestCase):

    def setUp(self):
        self.user1 = get_user_model().objects.get_or_create(username="test1")[0]
        self.user2 = get_user_model().objects.get_or_create(username="test2")[0]

    def tearDown(self):
        get_user_model().objects.all().delete()
        Person.all_objects.all().delete()

    def test_save_person_without_user(self):
        person = Person()
        person.save()

        self.assertEqual(person.created_by, None)
        self.assertEqual(person.updated_by, None)

    def test_save_person_with_user(self):
        person = Person(name='john doe')
        person.save(current_user=self.user1)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user1)

        person_from_db = Person.objects.get(name='john doe')
        self.assertEqual(person_from_db.created_by, self.user1)
        self.assertEqual(person_from_db.updated_by, self.user1)

    def test_create_person_without_user(self):
        person = Person.objects.create(name='john doe')

        self.assertEqual(person.created_by, None)
        self.assertEqual(person.updated_by, None)

        person_from_db = Person.objects.get(name='john doe')
        self.assertEqual(person_from_db.created_by, None)
        self.assertEqual(person_from_db.updated_by, None)

    def test_create_person_with_user(self):
        person = Person.objects.create(name='john doe', current_user=self.user1)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user1)

        person_from_db = Person.objects.get(name='john doe')
        self.assertEqual(person_from_db.created_by, self.user1)
        self.assertEqual(person_from_db.updated_by, self.user1)

    def test_update_person_with_user(self):
        person = Person.objects.create(name='john doe', current_user=self.user1)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user1)

        person_from_db = Person.objects.get(name='john doe')
        self.assertEqual(person_from_db.created_by, self.user1)
        self.assertEqual(person_from_db.updated_by, self.user1)

        person.save(current_user=self.user2)

        self.assertEqual(person.created_by, self.user1)
        self.assertEqual(person.updated_by, self.user2)

        person_from_db = Person.objects.get(name='john doe')
        self.assertEqual(person_from_db.created_by, self.user1)
        self.assertEqual(person_from_db.updated_by, self.user2)


class TestBasisSerializer(APITestCase):

    def setUp(self):
        self.user1 = get_user_model().objects.get_or_create(username="test1")[0]
        self.user2 = get_user_model().objects.get_or_create(username="test2")[0]

        self.factory = APIRequestFactory()
        self.request_post = self.factory.post('/', {'name': 'username 123'})
        self.view_post = BasisModelViewSet.as_view({'post': 'create'})

        self.request_put = self.factory.put('/', {'name': 'username 321'})
        self.view_put = BasisModelViewSet.as_view({'put': 'update'})

    def test_created_by(self):
        force_authenticate(self.request_post, user=self.user1)
        response = self.view_post(self.request_post)

        self.assertEqual(response.data['created_by'], self.user1.pk)
        self.assertEqual(response.data['updated_by'], self.user1.pk)

    def test_updated_by(self):
        user = Person(name="username 123")
        user.save(current_user=self.user1)
        id = user.pk

        force_authenticate(self.request_put, user=self.user2)
        response = self.view_put(self.request_put, pk=id)

        self.assertNotEqual(response.data['created_by'], self.user2.pk)
        self.assertEqual(response.data['updated_by'], self.user2.pk)



from rest_framework import serializers


class BasisSerializer(serializers.ModelSerializer):
    def save(self, **kwargs):
        request = self.context['request']
        kwargs['current_user'] = request.user
        super(BasisSerializer, self).save(**kwargs)



from datetime import datetime

from django.db import models
from django.conf import settings
from django.utils import timezone

from .compat import AUTH_USER_MODEL
from .managers import PersistentModelManager


def _now():
    if settings.USE_TZ:
        return timezone.now()
    return datetime.now()


class TimeStampModel(models.Model):
    created_at = models.DateTimeField(default=_now, editable=False)
    updated_at = models.DateTimeField(default=_now, editable=False)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self.updated_at = _now()
        super(TimeStampModel, self).save(*args, **kwargs)


class PersistentModel(models.Model):
    deleted = models.BooleanField(default=False, editable=False)

    objects = PersistentModelManager()
    all_objects = models.Manager()

    class Meta:
        abstract = True

    def delete(self, using=None, force=False):
        if force:
            super(PersistentModel, self).delete(using)
        else:
            self.deleted = True
            self.save()

    def restore(self, *args, **kwargs):
        self.deleted = False
        self.save(*args, **kwargs)


class BasisModel(TimeStampModel, PersistentModel):
    created_by = models.ForeignKey(AUTH_USER_MODEL, null=True, default=None, editable=False,
                                   related_name="%(class)s_created")
    updated_by = models.ForeignKey(AUTH_USER_MODEL, null=True, default=None, editable=False,
                                   related_name="%(class)s_updated")

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        try:
            kwargs['current_user'] = self.current_user
        except AttributeError:
            pass
        self.__set_user(kwargs)
        super(BasisModel, self).save(*args, **kwargs)

    def __set_user(self, kwargs):
        if 'current_user' in kwargs:
            current_user = kwargs.pop("current_user")

            if not self.id:
                self.created_by = current_user

            self.updated_by = current_user



# -*- coding: utf8 -*-
import os
from setuptools import setup, find_packages

os.environ['DJANGO_SETTINGS_MODULE'] = 'tests.settings'


def _read_long_description():
    try:
        import pypandoc
        return pypandoc.convert('README.md', 'rst', format='markdown')
    except Exception:
        return None


setup(
    name="django-basis",
    version='0.3.5',
    url='http://github.com/frecar/django-basis',
    author='Fredrik Carlsen',
    author_email='fredrik@carlsen.io',
    description='Simple reusable django app for basic model functionality',
    long_description=_read_long_description(),
    packages=find_packages(exclude='tests'),
    tests_require=[
        'django>=1.4',
    ],
    license='MIT',
    test_suite='runtests.runtests',
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Natural Language :: English",
    ]
)



from rest_framework import viewsets
from .models import Person
from .serializers import PersonSerializer


class BasisModelViewSet(viewsets.ModelViewSet):
    queryset = Person.objects.all()
    serializer_class = PersonSerializer
